/*
 * Copyright (c) 2022, 2023 Jonathan Schleifer <js@nil.im>
 *
 * All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License version 3.0 only,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3.0 along with this program. If not, see
 * <https://www.gnu.org/licenses/>.
 */

#import "O3DWinAPIWindow.h"

#ifdef OF_WINDOWS
# include <winuser.h>

@implementation O3DWinAPIWindow
@synthesize hWnd = _hWnd;

- (instancetype)initWithSize: (OFSize)size
{
	self = [super init];

	@try {
		_wndClass = (WNDCLASS){
			.lpfnWndProc = (WNDPROC)DefWindowProc,
			.hInstance = GetModuleHandle(NULL),
			.lpszClassName = "O3DMainWindowClass"
		};

		if (!RegisterClass(&_wndClass))
			@throw [OFInitializationFailedException
			    exceptionWithClass: self.class];

		if ((_hWnd = CreateWindowEx(WS_EX_APPWINDOW,
		    _wndClass.lpszClassName, "O3DMainWindow", WS_OVERLAPPED,
		    CW_USEDEFAULT, CW_USEDEFAULT, size.width, size.height,
		    NULL, NULL, _wndClass.hInstance, NULL)) == NULL)
			@throw [OFInitializationFailedException
			    exceptionWithClass: self.class];

		ShowWindow(_hWnd, SW_NORMAL);
		UpdateWindow(_hWnd);
	} @catch (id e) {
		[self release];
		@throw e;
	}

	return self;
}

- (instancetype)init
{
	OF_INVALID_INIT_METHOD
}

- (void)dealloc
{
	if (_hWnd != NULL)
		OFEnsure(DestroyWindow(_hWnd));

	if (_wndClass.lpszClassName != NULL)
		OFEnsure(UnregisterClass(
		    _wndClass.lpszClassName, _wndClass.hInstance));

	[super dealloc];
}
@end
#endif
